package com.emato.cus.supervise.biz.acqInventoryInfo;

import com.emato.cus.supervise.biz.CusWmsTask;
import com.emato.cus.supervise.biz.OmsDataImpl;
import com.emato.cus.supervise.constant.EmtRecordTimeEnum;
import com.emato.cus.supervise.domain.WmsAcqInventoryInfo;
import com.emato.cus.supervise.domainCus.CusAcqInventoryInfo03;
import com.emato.cus.supervise.domainOms.OmsMerchInfo;
import com.emato.cus.supervise.domainOms.OmsProductInfo;
import com.emato.cus.supervise.domainOmsShop.OmsShopAcqInventoryInfo;
import com.emato.cus.supervise.util.GsonUtil;
import com.emato.cus.supervise.util.LocalDateTimeUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 企业库存信息业务
 * 海关全量抓取
 * 以sku和local_ems_no园区账册编号作为联合主键
 *
 * @author zengjunlin
 * @version 1.0
 * 2018-02-01 16:29
 */
@Component
public class AcqInventoryInfoBiz implements CusWmsTask {

    private static final Logger logger = LoggerFactory.getLogger(AcqInventoryInfoBiz.class);


    @Autowired
    private AcqInventoryInfoRecordTime acqInventoryInfoRecordTime;

    @Autowired
    private AcqInventoryInfoWmsData acqInventoryInfoWmsData;

    @Autowired
    private AcqInventoryInfoCusData acqInventoryInfoCusData;

    @Autowired
    private AcqInventoryInfoDataConversion acqInventoryInfoDataConversion;

    @Autowired
    private OmsDataImpl omsDataImpl;


    @Autowired
    private AcqInventoryInfoOmsShopData acqInventoryInfoOmsShopData;
    @Autowired
    private AcqInventoryInfoOmsShopDataConversion acqInventoryInfoOmsShopDataConversion;


    private static final String DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.I_ACQ_INVENTORY_INFO.getCode() + "]" + EmtRecordTimeEnum.I_ACQ_INVENTORY_INFO.getName()
            + "，[" + EmtRecordTimeEnum.OMS_SHOP_ACQ_INVENTORY_INFO.getCode() + "]" + EmtRecordTimeEnum.OMS_SHOP_ACQ_INVENTORY_INFO.getName();

    private static final String WMS_DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.I_ACQ_INVENTORY_INFO.getCode() + "]" + EmtRecordTimeEnum.I_ACQ_INVENTORY_INFO.getName();
    private static final String SHOP_DATA_TYPE_INFO = "[" + EmtRecordTimeEnum.OMS_SHOP_ACQ_INVENTORY_INFO.getCode() + "]" + EmtRecordTimeEnum.OMS_SHOP_ACQ_INVENTORY_INFO.getName();



    /**
     * 任务入口
     */
    @Override
    public void task() {
        // 获取扫描时间
        String oldThisTime = acqInventoryInfoRecordTime.getEmtRecordTime();
        if (StringUtils.isBlank(oldThisTime)) {
            logger.info("--- 【" + DATA_TYPE_INFO + "】未获取到本次扫描时间, 本次扫描提前结束！！！");
            //下次扫描时间
            acqInventoryInfoRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN));
            return;

        }

        // 扫描WMS 数据
        List<WmsAcqInventoryInfo> wmsList = acqInventoryInfoWmsData.getWmsData(oldThisTime);
        // 执行wms相关操作
        boolean wmsOp = true;
        if (wmsList == null || wmsList.size() == 0) {
            logger.info("--- 【" + WMS_DATA_TYPE_INFO + "】获取到本次扫描WMS数据为空, 不执行写数据操作。");
            // 不执行WMS相关操作
            wmsOp = false;
            //下次扫描时间
            /*acqInventoryInfoRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;*/
        }


        // req_20200430001 门店库存海关监管
        /*List<OmsShopAcqInventoryInfo> omsShopList = acqInventoryInfoOmsShopData.getOmsShopData(oldThisTime);
        boolean omsShopOp = true;
        if (omsShopList == null || omsShopList.size() == 0) {
            logger.info("--- 【" + SHOP_DATA_TYPE_INFO + "】获取到本次扫描数据为空, 不执行写数据操作.");
            omsShopOp = false;
        }*/

        //if (!wmsOp && !omsShopOp) {
        /*if (!wmsOp) {
            logger.info("--- 【" + WMS_DATA_TYPE_INFO + "】获取到本次扫描数据为空, 不执行写数据操作, 提前结束");
            //下次扫描时间
            acqInventoryInfoRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;
        }*/

        if (wmsOp) {
            operateWms(wmsList, oldThisTime);
        }

        /*if (omsShopOp) {
            operateOmsShop(omsShopList, oldThisTime);
        }*/

        // 更新扫描时间
        boolean bl = acqInventoryInfoRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                oldThisTime);
    }


    /**
     * 写入WMS数据
     * @param wmsList
     * @param oldThisTime
     */
    private void operateWms(List<WmsAcqInventoryInfo> wmsList, String oldThisTime) {
        List<String> merchSnList = Lists.newArrayList();
        wmsList.forEach(i -> merchSnList.add(i.getMerchSn()));

        // 查询oms系统数据。
        List<OmsMerchInfo> omsList = omsDataImpl.getOmsMerchInfoData(merchSnList);

        if (omsList == null || omsList.isEmpty()) {
            logger.error("oms扫描到的数据" + GsonUtil.toJson(wmsList));
            logger.info("--- 【" + WMS_DATA_TYPE_INFO + "】获取到本次扫描OMS数据为空, 不执行写数据操作, 本次扫描提前结束！！！");
            //下次扫描时间
            acqInventoryInfoRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;
        }

        List<String> skuList = Lists.newArrayList();
        wmsList.forEach(i -> skuList.add(i.getSkuNo()));

        List<OmsProductInfo> productList = omsDataImpl.getOmsProductData(skuList);

        if (productList == null || productList.isEmpty()) {
            logger.error("oms扫描到的产品数据" + GsonUtil.toJson(productList));
            logger.info("--- 【" + DATA_TYPE_INFO + "】获取到OMS产品数据为空, 不执行写数据操作, 本次扫描提前结束！！！");
            //下次扫描时间
            acqInventoryInfoRecordTime.updateWmsRecordTime(LocalDateTimeUtils.formatNow(LocalDateTimeUtils.DATA_TIME_HYPHEN),
                    oldThisTime);
            return;
        }

        Map params = Maps.newHashMap();
        params.put("wmsList", wmsList);
        params.put("omsList", omsList);
        params.put("productInfoList", productList);

        //oms和wms数据，转成海关数据
        List<CusAcqInventoryInfo03> cusAcqInventoryInfo03List = acqInventoryInfoDataConversion.conversion(params);

        // 有则更新，无则新增
        // 写CUS 数据
        int res = acqInventoryInfoCusData.writeCus(cusAcqInventoryInfo03List);
    }


    /**
     * 写入OMS shop
     * @param omsShopList
     * @param oldThisTim
     */
    /*@Deprecated
    private void operateOmsShop(List<OmsShopAcqInventoryInfo> omsShopList, String oldThisTim) {
        Map params = Maps.newHashMap();
        params.put("omsShopList",omsShopList);

        //将oms shop数据转换cus
        List<CusAcqInventoryInfo03> cusList = acqInventoryInfoOmsShopDataConversion.conversion(params);

        // 写CUS 数据
        int res = acqInventoryInfoCusData.writeCus(cusList);
    }*/


}
