package com.emato.cus.supervise.config.ds.primary;

import com.emato.cus.supervise.util.YmlUtils;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.PropertyResolver;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

/**
 * 多数据源配置
 *
 * 数据源配置
 *
 * 指定mybatis-config.xml, 和要扫描的别名包
 *
 * @author Scott Chen
 * @date 2017/4/22
 */
@Configuration
public class PrimaryDataSourceConfig {

    // yml方式配置
    public static final String MYBATIS_MAPPER_CONFIG_YML_PATH = "application.yml";

    @Bean
    public DataSource primaryDataSource() {
        return primaryDataSourceProperties().initializeDataSourceBuilder().build();
    }

    @Bean
    @ConfigurationProperties(prefix = "spring.datasource.primary")
    public DataSourceProperties primaryDataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean
    public SqlSessionFactory primarySqlSessionFactory(DataSource primaryDataSource) throws Exception  {
        SqlSessionFactoryBean sessionFactoryBean = new SqlSessionFactoryBean();
        sessionFactoryBean.setDataSource(primaryDataSource);
        //Mybatis XML配置文件
        sessionFactoryBean.setConfigLocation(new PathMatchingResourcePatternResolver()
                .getResource(configLocationByYml()));
        sessionFactoryBean.setMapperLocations(new PathMatchingResourcePatternResolver()
                .getResources(mapperLocationsByYml()));
        sessionFactoryBean.setTypeAliasesPackage(typeAliasesByYml());
        return sessionFactoryBean.getObject();
    }

    @Bean
    public DataSourceTransactionManager primaryTransactionManager(DataSource primaryDataSource) {
        return new DataSourceTransactionManager(primaryDataSource);
    }


    @Bean
    public SqlSessionTemplate primarySqlSessionTemplate(SqlSessionFactory primarySqlSessionFactory) {
        return new SqlSessionTemplate(primarySqlSessionFactory);
    }


    private String configLocationByYml() {
        String path = MYBATIS_MAPPER_CONFIG_YML_PATH;
        String key = "mybatis.primary.config-location";
        return YmlUtils.instance().getValue(key, path);
    }

    private String mapperLocationsByYml() {
        String path = MYBATIS_MAPPER_CONFIG_YML_PATH;
        String key = "mybatis.primary.mapper-locations-config";
        return YmlUtils.instance().getValue(key, path);
    }

    private String typeAliasesByYml() {
        String path = MYBATIS_MAPPER_CONFIG_YML_PATH;
        String key = "mybatis.primary.type-aliases-package";
        return YmlUtils.instance().getValue(key, path);
    }

}
